'use strict';

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var path = require('path');
var os = require('os');
var fs = require('fs');

var _require = require('../../helpers/_exec'),
    execPromise = _require.execPromise;

var temp = require("temp");
var AWS = require('aws-sdk');
var sha256 = require('js-sha256').sha256;

var _require2 = require('util'),
    promisify = _require2.promisify;

var mkdir = promisify(temp.mkdir); // (A)
temp.track();

module.exports = {
    description: "installs a templated plugin or a seed, without writing in deps",
    builder: function builder(yargs) {
        return yargs.option('type', {
            // describe: '',
            default: "ipfs"
        }).option('invalidate', {
            // describe: '',
            default: true
        }).option('bucket', {
            // describe: '',
            default: "liquidapps.artifacts"
        }).option('prefix', {
            // describe: '',
            default: "boxes/"
        }).option('update-mapping', {
            // describe: '',
            default: false
        }).example('$0 deploy box');
    },
    command: 'box',
    handler: function () {
        var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(args) {
            var stagingPath, packageName, stdout, inputPath, zeusBoxJsonPath, zeusBoxJson, ignoreList, hooks, moddate, uri, hash, s3, data, binaryData, s3Key, res, ipfsout, urls, mappingFile, boxes;
            return regeneratorRuntime.wrap(function _callee$(_context) {
                while (1) {
                    switch (_context.prev = _context.next) {
                        case 0:
                            _context.next = 2;
                            return mkdir('zeus');

                        case 2:
                            stagingPath = _context.sent;
                            packageName = path.resolve('.').split(path.sep).pop();
                            stdout = void 0;
                            inputPath = path.resolve('.');


                            console.log('staging in ' + stagingPath);
                            zeusBoxJsonPath = path.join(inputPath, 'zeus-box.json');

                            if (fs.existsSync(zeusBoxJsonPath)) {
                                _context.next = 11;
                                break;
                            }

                            temp.cleanupSync();
                            throw new Error('zeus-box.json not found');

                        case 11:
                            zeusBoxJson = JSON.parse(fs.readFileSync(zeusBoxJsonPath));
                            ignoreList = [];

                            if (zeusBoxJson.ignore) ignoreList = zeusBoxJson.ignore.filter(function (a) {
                                return a != 'zeus-box.json';
                            }).map(function (a) {
                                return a;
                            });

                            if (fs.existsSync('.gitignore') && !args.noIgnore) {
                                ignoreList = [].concat(_toConsumableArray(ignoreList), _toConsumableArray(fs.readFileSync('.gitignore').toString().split('\n').filter(function (a) {
                                    return a.trim() != '';
                                })));
                            }
                            ignoreList = [].concat(_toConsumableArray(ignoreList), ['.git']);
                            // copy to dest
                            _context.next = 18;
                            return execPromise((process.env.RSYNC || 'rsync') + ' -a ' + path.join(inputPath) + ' ' + stagingPath + ' ' + ignoreList.map(function (a) {
                                return '--exclude ' + a;
                            }).join(' '), {
                                cwd: path.resolve('.')
                            });

                        case 18:
                            stdout = _context.sent;


                            // load zeus-box.json
                            hooks = zeusBoxJson.hooks;

                            // run build script

                            if (!(hooks && hooks['post-stage'])) {
                                _context.next = 25;
                                break;
                            }

                            console.log('running post stage');
                            _context.next = 24;
                            return execPromise(hooks['post-stage'], {
                                cwd: stagingPath
                            });

                        case 24:
                            stdout = _context.sent;

                        case 25:
                            _context.next = 27;
                            return execPromise('find . -not -name "." -exec date -I -r \'{}\' \\; | sort -rn | head -1', { cwd: stagingPath });

                        case 27:
                            moddate = _context.sent;

                            moddate = moddate.trim() + " 00:00";
                            _context.next = 31;
                            return execPromise((process.env.ZIP || 'zip') + ' -X -r ./box.zip .', { cwd: stagingPath });

                        case 31:
                            stdout = _context.sent;

                            // console.log("moddate",moddate)

                            uri = "";
                            _context.t0 = args.type;
                            _context.next = _context.t0 === 's3' ? 36 : _context.t0 === 'ipfs' ? 49 : 49;
                            break;

                        case 36:
                            args.invalidate = false;
                            s3 = new AWS.S3({ apiVersion: '2006-03-01' });
                            data = fs.readFileSync(path.join(stagingPath, './box.zip'));

                            hash = sha256(data);

                            binaryData = new Buffer(data, 'binary');

                            if (!(args.bucket.length == 0)) {
                                _context.next = 43;
                                break;
                            }

                            throw new Error("must pass bucket when uploading to S3");

                        case 43:
                            s3Key = '' + args.prefix + hash + '.zip';
                            _context.next = 46;
                            return s3.putObject({
                                Bucket: args.bucket,
                                Key: s3Key,
                                ACL: 'public-read',
                                Body: binaryData
                            }).promise();

                        case 46:
                            res = _context.sent;

                            uri = 'https://s3.us-east-2.amazonaws.com/' + args.bucket + '/' + s3Key;
                            return _context.abrupt('break', 57);

                        case 49:
                            _context.next = 51;
                            return execPromise((process.env.IPFS || 'ipfs') + ' add ./box.zip', { cwd: stagingPath });

                        case 51:
                            ipfsout = _context.sent;

                            hash = ipfsout.split(' ')[1];
                            uri = 'ipfs://' + hash;
                            _context.next = 56;
                            return execPromise('touch -d "' + moddate + '" ./box.zip', { cwd: stagingPath });

                        case 56:
                            stdout = _context.sent;

                        case 57:

                            console.log('box deployed to ' + uri);
                            // run post script

                            // invalidate endpoints:

                            if (!(args.invalidate && !(process.env.SKIP_IPFS_GATEWAY == 'true'))) {
                                _context.next = 63;
                                break;
                            }

                            console.log('invalidating ipfs...');
                            urls = ['https://ipfs.io/ipfs/' + hash, 'https://cloudflare-ipfs.com/ipfs/' + hash, 'https://ipfs.io/ipfs/' + hash, 'https://cloudflare-ipfs.com/ipfs/' + hash];
                            _context.next = 63;
                            return Promise.race(urls.map(function (url) {
                                return execPromise((process.env.CURL || 'curl') + ' --silent --output /dev/null ' + url, {});
                            }));

                        case 63:

                            // console.log('cleaning up');
                            temp.cleanupSync();
                            // var archive = `https://github.com/${}/${}/archive/master.zip`; 
                            // https://github.com/zeit/serve/archive/master.zip
                            if (args['update-mapping']) {
                                console.log("updating local mapping", packageName);

                                mappingFile = path.resolve(__dirname, "../../mapping.js");
                                boxes = require("../../mapping");

                                boxes[packageName] = uri;
                                fs.writeFileSync(path.resolve(mappingFile), 'module.exports = ' + JSON.stringify(boxes, null, 2) + ';');
                            }
                            console.log('done.');
                            process.exit();
                            // resolve github 3rd party boxes
                            // resolve ipfs boxes
                            // resolve "onchain" boxes

                        case 67:
                        case 'end':
                            return _context.stop();
                    }
                }
            }, _callee, undefined);
        }));

        return function handler(_x) {
            return _ref.apply(this, arguments);
        };
    }()
};