'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var path = require('path');
var os = require('os');
var fs = require('fs');

var _require = require('../helpers/_exec'),
    execPromise = _require.execPromise,
    emojMap = _require.emojMap;

var temp = require("temp");

var _require2 = require('util'),
    promisify = _require2.promisify;

var mkdir = promisify(temp.mkdir); // (A)
temp.track();
var isDirectory = function isDirectory(source) {
    return fs.lstatSync(source).isDirectory();
};
var getDirectories = function getDirectories(source) {
    return fs.readdirSync(source).map(function (name) {
        return path.join(source, name);
    }).filter(isDirectory);
};
function copyFileSync(source, target, simulate, newIgnoreList, rootPath) {

    var targetFile = target;

    //if target is a directory a new file with the same name will be created
    if (fs.existsSync(target)) {
        if (fs.lstatSync(target).isDirectory()) {
            targetFile = path.join(target, path.basename(source));
        } else {
            var targetBuf = fs.readFileSync(target);
            var sourceBuf = fs.readFileSync(source);
            if (!sourceBuf.equals(targetBuf)) {
                throw new Error('cannot override ' + target);
            } else {
                return;
            }
        }
    }
    newIgnoreList.push(target.substr(rootPath.length));
    if (!simulate) {
        var stats = fs.lstatSync(source);
        fs.writeFileSync(targetFile, fs.readFileSync(source));
        fs.chmodSync(targetFile, stats.mode);
    }
}
var mapping = require("../mapping");

function copyFolderRecursiveSync(source, target, simulate, ignoreList, newIgnoreList, rootPath) {
    var files = [];

    //check if folder needs to be created or integrated
    var targetFolder = target;
    if (!simulate && !fs.existsSync(targetFolder)) {
        fs.mkdirSync(targetFolder);
    }
    var stats = fs.lstatSync(source);
    //copy

    if (stats.isDirectory()) {
        files = fs.readdirSync(source);
        files.forEach(function (file) {
            var curSource = path.join(source, file);
            var curTarget = path.join(targetFolder, file);
            if (ignoreList.indexOf(curSource) > -1) {
                // console.log("ignoring ",curSource);
                return;
            }
            var fstats = fs.lstatSync(curSource);
            if (fstats.isDirectory()) {
                copyFolderRecursiveSync(curSource, curTarget, simulate, ignoreList, newIgnoreList, rootPath);
            } else {

                // add to gitignore
                copyFileSync(curSource, curTarget, simulate, newIgnoreList, rootPath);
            }
        });
    }
}

var runHook = function () {
    var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(hookName, args, zeusBoxJson, location, opts) {
        var hooks, stdout, hookFile;
        return regeneratorRuntime.wrap(function _callee$(_context) {
            while (1) {
                switch (_context.prev = _context.next) {
                    case 0:
                        opts = opts || {
                            cwd: path.resolve('.')
                        };

                        if (!zeusBoxJson.hooks) {
                            _context.next = 8;
                            break;
                        }

                        hooks = zeusBoxJson.hooks;

                        if (!hooks[hookName]) {
                            _context.next = 8;
                            break;
                        }

                        console.log(emojMap.comet + (hookName.green + ' hook for box: ' + args.box.yellow));
                        _context.next = 7;
                        return execPromise(hooks[hookName], opts);

                    case 7:
                        stdout = _context.sent;

                    case 8:
                        // check for post-install hooks under ./extensions/commands/hook/post-install.js
                        hookFile = path.resolve('./extensions/commands/hook/' + hookName);

                        if (!fs.existsSync(hookFile + ".js")) {
                            _context.next = 12;
                            break;
                        }

                        _context.next = 12;
                        return require(hookFile).handler(args, zeusBoxJson, location, opts);

                    case 12:
                    case 'end':
                        return _context.stop();
                }
            }
        }, _callee, undefined);
    }));

    return function runHook(_x, _x2, _x3, _x4, _x5) {
        return _ref.apply(this, arguments);
    };
}();
var handler = function () {
    var _ref2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(args) {
        var globalCopyList = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
        var boxName, extractPath, stdout, inputPath, currentzeusBoxJson, createDir, newDir, protocol, dirPath, hash, ipfsout, file, dirs, zeusBoxJsonPath, zeusBoxJson, i, dep, unboxExt, params, ignoreList, newIgnoreList, currentGitIgnore, dirToDelete;
        return regeneratorRuntime.wrap(function _callee2$(_context2) {
            while (1) {
                switch (_context2.prev = _context2.next) {
                    case 0:
                        // resolve github system boxes
                        boxName = args.box;
                        // var repo = boxName.split('/')[0];
                        // var repo = boxName.split('/')[1];

                        if (mapping[boxName]) boxName = mapping[boxName];
                        _context2.next = 4;
                        return mkdir('zeus');

                    case 4:
                        extractPath = _context2.sent;
                        stdout = void 0;
                        currentzeusBoxJson = {
                            "ignore": ["README.md", "zeus-box.json", "zeus-config.js"],
                            "commands": {},
                            "hooks": {},
                            "dependencies": []
                        };
                        createDir = false;

                        if (!global.currentzeusBoxJson) {
                            _context2.next = 12;
                            break;
                        }

                        currentzeusBoxJson = global.currentzeusBoxJson;
                        _context2.next = 25;
                        break;

                    case 12:
                        if (!fs.existsSync(path.resolve('./zeus-box.json'))) {
                            _context2.next = 17;
                            break;
                        }

                        currentzeusBoxJson = JSON.parse(fs.readFileSync(path.resolve('./zeus-box.json')));
                        if (!currentzeusBoxJson.dependencies) currentzeusBoxJson.dependencies = [];

                        _context2.next = 25;
                        break;

                    case 17:
                        if (!args.createDir) {
                            _context2.next = 25;
                            break;
                        }

                        createDir = true;
                        newDir = path.resolve('./', args.box);

                        if (!fs.existsSync(newDir)) {
                            _context2.next = 22;
                            break;
                        }

                        throw new Error("path already exists", newDir);

                    case 22:
                        fs.mkdirSync(newDir);
                        process.chdir(newDir);
                        args.createDir = false;

                    case 25:
                        if (!(currentzeusBoxJson.dependencies.indexOf(args.box) != -1)) {
                            _context2.next = 27;
                            break;
                        }

                        return _context2.abrupt('return');

                    case 27:

                        if (!args.no_sample) {
                            console.log(emojMap.gear + 'Unboxing:', args.box.yellow);
                        }

                        currentzeusBoxJson.dependencies.push(args.box);
                        global.currentzeusBoxJson = currentzeusBoxJson;

                        if (!(boxName.indexOf('://') > 0 && boxName.indexOf('://') < 10)) {
                            _context2.next = 58;
                            break;
                        }

                        protocol = boxName.split('://')[0];
                        _context2.next = 34;
                        return mkdir('zeus');

                    case 34:
                        dirPath = _context2.sent;

                        inputPath = path.join(dirPath, 'archive.zip');

                        _context2.t0 = protocol;
                        _context2.next = _context2.t0 === 'ipfs' ? 39 : _context2.t0 === 'http' ? 48 : _context2.t0 === 'https' ? 48 : _context2.t0 === 'file' ? 52 : 55;
                        break;

                    case 39:
                        hash = boxName.split('://')[1];

                        if (process.env.SKIP_IPFS_GATEWAY) {
                            _context2.next = 44;
                            break;
                        }

                        boxName = 'https://cloudflare-ipfs.com/ipfs/' + hash;
                        _context2.next = 48;
                        break;

                    case 44:
                        _context2.next = 46;
                        return execPromise((process.env.IPFS || 'ipfs') + ' cat /ipfs/' + hash + ' > ' + inputPath);

                    case 46:
                        ipfsout = _context2.sent;
                        return _context2.abrupt('break', 56);

                    case 48:
                        _context2.next = 50;
                        return execPromise((process.env.CURL || 'curl') + ' ' + boxName + ' -L -o ' + inputPath, {});

                    case 50:
                        stdout = _context2.sent;
                        return _context2.abrupt('break', 56);

                    case 52:
                        file = boxName.split('://')[1];

                        inputPath = file;
                        return _context2.abrupt('break', 56);

                    case 55:
                        throw new Error("not supported yet. pass archive url:" + boxName);

                    case 56:
                        _context2.next = 59;
                        break;

                    case 58:
                        throw new Error('not supported yet. pass archive url ' + args.box);

                    case 59:
                        _context2.next = 61;
                        return function () {
                            return new Promise(function (resolve, reject) {
                                var extractor = require('unzip').Extract({ path: extractPath });
                                extractor.on('close', resolve);
                                extractor.on('error', reject);
                                fs.createReadStream(inputPath).pipe(extractor);
                            });
                        }();

                    case 61:

                        // console.log(stdout);
                        dirs = getDirectories(extractPath);

                        if (!(dirs.length != 1)) {
                            _context2.next = 65;
                            break;
                        }

                        temp.cleanupSync();
                        throw new Error('unknown archive format');

                    case 65:
                        extractPath = dirs[0];
                        zeusBoxJsonPath = path.join(extractPath, 'zeus-box.json');

                        if (fs.existsSync(zeusBoxJsonPath)) {
                            _context2.next = 70;
                            break;
                        }

                        temp.cleanupSync();
                        throw new Error('zeus-box.json not found');

                    case 70:
                        zeusBoxJson = JSON.parse(fs.readFileSync(zeusBoxJsonPath));

                        // load zeus-box.json

                        if (!zeusBoxJson.dependencies) {
                            _context2.next = 88;
                            break;
                        }

                        if (zeusBoxJson.dependencies.filter(function (pkg) {
                            return currentzeusBoxJson.dependencies.indexOf(pkg) == -1;
                        }).length) console.log(emojMap.gear + "Unboxing:", zeusBoxJson.dependencies.filter(function (pkg) {
                            return currentzeusBoxJson.dependencies.indexOf(pkg) == -1;
                        }).map(function (a) {
                            return a.yellow;
                        }).join(','));
                        // todo: load extension unbox if exists
                        i = 0;

                    case 74:
                        if (!(i < zeusBoxJson.dependencies.length)) {
                            _context2.next = 88;
                            break;
                        }

                        dep = zeusBoxJson.dependencies[i];
                        unboxExt = path.resolve('./extensions/commands/unbox.js');
                        params = _extends({}, args, { box: dep, no_sample: true });

                        if (!fs.existsSync(unboxExt + ".js")) {
                            _context2.next = 83;
                            break;
                        }

                        _context2.next = 81;
                        return require(unboxExt).handler(params, globalCopyList);

                    case 81:
                        _context2.next = 85;
                        break;

                    case 83:
                        _context2.next = 85;
                        return handler(params, globalCopyList);

                    case 85:
                        i++;
                        _context2.next = 74;
                        break;

                    case 88:
                        _context2.next = 90;
                        return runHook('post-unpack', args, zeusBoxJson, extractPath, {
                            cwd: extractPath
                        });

                    case 90:

                        // delete files in box ignore list
                        ignoreList = [];
                        newIgnoreList = [];

                        if (zeusBoxJson.ignore) ignoreList = zeusBoxJson.ignore.map(function (a) {
                            return path.join(extractPath, a);
                        });
                        // compare if any file already exists
                        // console.log('copying staged files');
                        copyFolderRecursiveSync(extractPath, path.resolve('.'), false, ignoreList, newIgnoreList, path.resolve('.'));
                        globalCopyList.forEach(function (a) {
                            if (newIgnoreList.indexOf(a) == -1) newIgnoreList.push(a);
                        });

                        // if(zeusBoxJson.ignore)
                        //     ignoreList = zeusBoxJson.ignore.map(a=>a);
                        // // copy to dest
                        // stdout = await execPromise(`${process.env.RSYNC || 'rsync'} -a ${path.join(extractPath,'/')} . ${ignoreList.map(a=>`--exclude ${a}`).join(' ')}`,{
                        //     cwd:path.resolve('.')
                        // });
                        // console.log(stdout);
                        if (fs.existsSync('.gitignore') && !args.noIgnore) {
                            currentGitIgnore = fs.readFileSync('.gitignore').toString();

                            currentGitIgnore += '\n';
                            currentGitIgnore += newIgnoreList.map(function (a) {
                                return a.replace(/\\/g, '/');
                            }).filter(function (a) {
                                return a.indexOf('node_modules/') == -1 && a.indexOf('extensions/') == -1;
                            }).map(function (a) {
                                return a.substring(1);
                            }).filter(function (a) {
                                return a !== ".gitignore";
                            }).join('\n');
                            fs.writeFileSync('.gitignore', currentGitIgnore);
                        }
                        fs.writeFileSync(path.resolve('./zeus-box.json'), JSON.stringify(currentzeusBoxJson, null, 2));

                        // run install script
                        _context2.prev = 97;
                        _context2.next = 100;
                        return runHook('post-install', args, zeusBoxJson, extractPath);

                    case 100:
                        if (!(args.test && !args.no_sample)) {
                            _context2.next = 106;
                            break;
                        }

                        console.log("testing");
                        _context2.next = 104;
                        return execPromise((process.env.ZEUS_CMD || 'zeus') + ' test', {
                            cwd: path.resolve('.')
                        });

                    case 104:
                        stdout = _context2.sent;

                        console.log(stdout);

                    case 106:
                        _context2.next = 113;
                        break;

                    case 108:
                        _context2.prev = 108;
                        _context2.t1 = _context2['catch'](97);

                        // delete all files in  newIgnoreList
                        globalCopyList.forEach(function (f) {
                            try {
                                fs.unlinkSync(f);
                            } catch (e) {}
                        });
                        if (args.createDir) {
                            dirToDelete = path.resolve('.');

                            process.chdir(path.resolve('..'));
                            fs.rmdirSync(dirToDelete);
                        }
                        throw _context2.t1;

                    case 113:
                        if (!(args.no_sample || !zeusBoxJson.commands || Object.keys(zeusBoxJson.commands).length == 0)) {
                            _context2.next = 115;
                            break;
                        }

                        return _context2.abrupt('return');

                    case 115:
                        console.log(emojMap.ok + ('Done: ' + (createDir ? '\nEnter new directory:\n     cd ' + args.box : '') + '\nPlease try these sample commands:'));
                        console.log(Object.keys(zeusBoxJson.commands).map(function (label) {
                            return '     ' + emojMap.star + label.underline + ': ' + zeusBoxJson.commands[label].yellow;
                        }).join('\n'));
                        // resolve github 3rd party boxes
                        // resolve ipfs boxes
                        // resolve "onchain" boxes

                    case 117:
                    case 'end':
                        return _context2.stop();
                }
            }
        }, _callee2, undefined, [[97, 108]]);
    }));

    return function handler(_x6) {
        return _ref2.apply(this, arguments);
    };
}();
module.exports = {
    description: "installs a templated plugin or a seed",
    builder: function builder(yargs) {
        yargs.option('create-dir', {
            // describe: '',
            default: true
        }).option('test', {
            // describe: '',
            default: false
        }).alias('c', 'create-dir').example('$0 unbox seed --no-create-dir --no-test');
    },
    command: 'unbox <box>',
    handler: handler
};