'use strict';

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var _require = require('child_process'),
    exec = _require.exec,
    spawn = _require.spawn;

var path = require('path');
var fs = require('fs');
var emoji = require('node-emoji');
var colors = require('colors');

var execPromise2 = function execPromise2(cmd, options) {
    return new Promise(function (resolve, reject) {
        exec(cmd, options, function (err, stdout) {

            if (err) {
                err.stdout = stdout;
                return reject(err);
            }
            resolve(stdout);
        });
    });
};

var execPromise = function execPromise(cmd, options) {
    return new Promise(function (resolve, reject) {
        var childproc = exec(cmd, options, function (err, stdout) {
            // if (err) {
            //     err.stdout = stdout;
            //     return reject(err);
            // }
            // resolve(stdout);
        });
        options = options || {};
        var _options = options,
            printOutStream = _options.printOutStream,
            printErrStream = _options.printErrStream,
            printOutCB = _options.printOutCB,
            printErrCB = _options.printErrCB;

        var stdout = '';
        var stderr = '';
        // var stdoutLine = '';
        // var stderrLine = '';
        childproc.stdout.on('data', function (data) {
            var line = data.toString();
            if (printOutCB) line = printOutCB(line);
            if (printOutStream) printOutStream.write(line);
            stdout += line;
            //   stdoutLine+=line;
        });
        childproc.stderr.on('data', function (data) {
            var line = data.toString();
            if (printErrCB) line = printOutCB(line);
            if (printErrStream) printErrStream.write(line);
            stderr += data.toString();
            //   stderrLine += data.toString();
        });
        childproc.on('exit', function (code) {
            if (code != 0) {
                var err = new Error('exec failed');
                err.stdout = stdout;
                err.stderr = stderr;
                err.code = code;
                return reject(err);
            }
            return resolve(stdout);
            // console.log('child process exited with code ' + code.toString());
        });
    });
};

var lstatSync = fs.lstatSync,
    readdirSync = fs.readdirSync;

var _require2 = require('path'),
    join = _require2.join;

var isFile = function isFile(source) {
    return !lstatSync(source).isDirectory();
};
var getScripts = function getScripts(source) {
    return readdirSync(source).map(function (name) {
        return join(source, name);
    }).filter(isFile).filter(function (a) {
        return a.endsWith('.js');
    }).sort();
};

var execScripts = function () {
    var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(scriptsPath, argsFn, yargsArgs) {
        var scripts, i, script, newArgs, module;
        return regeneratorRuntime.wrap(function _callee$(_context) {
            while (1) {
                switch (_context.prev = _context.next) {
                    case 0:
                        global.yargsArgs = yargsArgs;
                        scripts = getScripts(scriptsPath);
                        i = 0;

                    case 3:
                        if (!(i < scripts.length)) {
                            _context.next = 14;
                            break;
                        }

                        script = scripts[i];
                        newArgs = argsFn(script);

                        if (newArgs) {
                            _context.next = 8;
                            break;
                        }

                        return _context.abrupt('continue', 11);

                    case 8:
                        module = require(script);
                        _context.next = 11;
                        return module.call.apply(module, [module].concat(_toConsumableArray(newArgs)));

                    case 11:
                        i++;
                        _context.next = 3;
                        break;

                    case 14:
                    case 'end':
                        return _context.stop();
                }
            }
        }, _callee, undefined);
    }));

    return function execScripts(_x, _x2, _x3) {
        return _ref.apply(this, arguments);
    };
}();

var manipulateTextAtPosition = function manipulateTextAtPosition(line, pos, length, fn) {
    return line.substr(0, pos) + fn(line.substr(pos, length)) + line.substr(pos + length);
};
var colorizeArr = function colorizeArr(txt, colorsArr) {
    return colorsArr.reduce(function (current, newOne) {
        return current[newOne];
    }, txt);
};
var colorizeMatch = function colorizeMatch(line, match) {
    for (var _len = arguments.length, colorsArr = Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
        colorsArr[_key - 2] = arguments[_key];
    }

    return !match ? line : colorizePos.apply(undefined, [line, match.index, match[0].length].concat(colorsArr));
};

var colorizeSubstr = function colorizeSubstr(line, str) {
    for (var _len2 = arguments.length, colorsArr = Array(_len2 > 2 ? _len2 - 2 : 0), _key2 = 2; _key2 < _len2; _key2++) {
        colorsArr[_key2 - 2] = arguments[_key2];
    }

    return line.indexOf(str) === -1 ? line : manipulateTextAtPosition(line, line.indexOf(str), str.length, function (txt) {
        return colorizeArr(txt, colorsArr);
    });
};

var colorizePos = function colorizePos(line, pos, length) {
    for (var _len3 = arguments.length, colorsArr = Array(_len3 > 3 ? _len3 - 3 : 0), _key3 = 3; _key3 < _len3; _key3++) {
        colorsArr[_key3 - 3] = arguments[_key3];
    }

    return manipulateTextAtPosition(line, pos, length, function (txt) {
        return colorizeArr(txt, colorsArr);
    });
};

var colorMap = {
    "message": "bold",
    "error": "red",
    "warning": "magenta",
    "note": "blue",
    "make": "gray",
    "cursor": "blue"
};
var emojMap = _defineProperty({
    "error": "warning",
    "warning": "pencil2",
    "ok": "heavy_check_mark",
    "note": "black_medium_small_square",
    "zap": "zap",
    "star": "star",
    "heart": "heart",
    "congratulations": "congratulations",
    "final_warning": "warning",
    "final_error": "skull_and_crossbones",
    "v": "v",
    "phone": "phone",
    "umbrella": "umbrella",
    "point_up": "point_up",
    "arrow_forward": "arrow_backward",
    "arrow_backward": "arrow_backward",
    "coffee": "coffee",
    "cloud": "cloud",
    "email": "email",
    "scissors": "scissors",
    "hotsprings": "hotsprings",
    "relaxed": "relaxed",
    "white_frowning_face": "white_frowning_face",
    "information_source": "information_source",
    "crossed_swords": "crossed_swords",
    "fleur_de_lis": "fleur_de_lis",
    "gear": "gear",
    "anchor": "anchor",
    "shamrock": "shamrock",
    "white_circle": "white_circle",
    "black_circle": "black_circle",
    "alembic": "alembic",
    "scales": "scales",
    "soccer": "soccer",
    "watch": "watch",
    "airplane": "airplane",
    "keyboard": "keyboard",
    "skull_and_crossbones": "skull_and_crossbones",
    "radioactive": "Radioactive Sign",
    "biohazard": "Biohazard Sign",
    "comet": "comet",
    "eight_pointed_black_star": "eight_pointed_black_star",
    "eight_spoked_asterisk": "eight_spoked_asterisk",
    "snowflake": "snowflake",
    "hourglass": "hourglass",
    "hammer_and_pick": "hammer_and_pick",
    "coffin": "Coffin",
    "bangbang": "bangbang",
    "sparkle": "sparkle",
    "sunny": "sunny",
    "heavy_multiplication_x": "heavy_multiplication_x",
    "writing_hand": "writing_hand",
    "interrobang": "interrobang",
    "black_medium_square": "black_medium_square",
    "white_medium_square": "white_medium_square"
}, 'zap', "zap");
Object.keys(emojMap).forEach(function (k) {
    return emojMap[k] = emoji.emojify(':' + emojMap[k] + ':').toLowerCase().replace(/[\-\ ]/g, '_');
});

module.exports = { execPromise: execPromise, execScripts: execScripts, emojMap: emojMap, colorizeMatch: colorizeMatch, colorizePos: colorizePos, colorizeArr: colorizeArr, manipulateTextAtPosition: manipulateTextAtPosition, colorMap: colorMap, colorizeSubstr: colorizeSubstr };